/*
 * linux/include/asm-arm/arch-vnc/time.h
 *
 * Copyright (c) 1997 Corel Computer Corp
 *
 */

#define MCLK_47_8

#define TIMER_CLEAR (timer_base + 3)
#define TIMER_CONTROL (timer_base + 2)
#define TIMER_VALUE (timer_base + 1)
#define TIMER_LOAD (timer_base)

volatile unsigned int * timer_base = (unsigned int*)(IO_BASE_ARM_CSR + TIMER_1_LOAD);

extern __inline__ unsigned long gettimeoffset (void)
{
	return 0;
}

extern __inline__ int reset_timer (void)
{  
    //*TIMER_CLEAR = 0;
    return 1;
}

extern __inline__ unsigned long setup_timer (void)
{
        unsigned int mode ; 
        unsigned char rev = (unsigned char) (board_id >> 12);
        /* use timer 1 on the PIT (irq 16) for time time base
           this is guaranteed to work for rev 3,4 & 5 boards */

        /* enable PIT timer */
        /* set for periodic (4) and LSB/MSB write (0x30) */
        outb(0x34, 0x43);
        outb((mSEC_10_from_14/6) & 0xFF, 0x40);
        outb((mSEC_10_from_14/6) >> 8, 0x40);

	*TIMER_LOAD = mSEC_10;
	mode = *TIMER_CONTROL ;
	mode |= TIMER_CONTROL_ENABLE | TIMER_AUTO_RELOAD;
	*TIMER_CONTROL = mode ;

        /* if one wishes a report on timer 1, include this section */
#if 1

        {
            int timer_val;

            rev = board_id >> 12;
            
            /* check to see if timer 3 is running */
            if (rev == 0xF)
            {
                int a,b,c;
                timer_base = (unsigned int*) (IO_BASE_ARM_CSR + TIMER_3_LOAD);
                *TIMER_CONTROL = TIMER_CONTROL_ENABLE | 0xC;
                *TIMER_LOAD = 0;
                a = *TIMER_VALUE;
                b = *TIMER_VALUE;
                c = *TIMER_VALUE;
                if (a==b || b==c)
                    rev = 4;
                else
                    rev = 5;
                *TIMER_CONTROL = 0;  /* turn it off */
                timer_base = (unsigned int*) (IO_BASE_ARM_CSR + TIMER_1_LOAD);
            }

            /* control reg, set to 7.8125ms */
            CMOS_WRITE(0x29, RTC_FREQ_SELECT);
            CMOS_READ(RTC_INTR_FLAGS);    /* read and clear pe bit */
            while (!(CMOS_READ(RTC_INTR_FLAGS) & RTC_PF));
            timer_val = *TIMER_VALUE;
            while (!(CMOS_READ(RTC_INTR_FLAGS) & RTC_PF));
            timer_val -= *TIMER_VALUE;
            if (timer_val<0) timer_val += *TIMER_LOAD;
            if (timer_val > ((mSEC_10 * 100)/128 + 35)
                || timer_val < ((mSEC_10 * 100)/128 - 35))
            {
                *TIMER_LOAD = (timer_val * 128) / 100;
                printk("CLOCK: Timer 1 is not crystal controlled. Loaded with %d\n",*TIMER_LOAD);
            }
            else
                printk("CLOCK: Timer 1 verified\n");
        }
        printk("CLOCK: board probe returned rev %x ",board_id);
        board_id = (board_id & ~0xF000) | (rev << 12);
        printk("and will report %x\n",board_id);

#endif
	/*
	 * Default the date to 1 Jan 1970 0:0:0
	 * You will have to run a time daemon to set the
	 * clock correctly at bootup
	 */
	return mktime(1970, 1, 1, 0, 0, 0);
}
